@@ -5,15 +5,15 @@ function(x, ...)
   cat("Call:\n")
   print(x$call)
 
-    cat("\nComponents partitioned by DME from residual var/covariance after OLS-b fit:\n\n")
+    cat("\nComponents partitioned by DME from residual var/covariance after OLS-fixed-effects fit:\n\n")
   for(ic in 1:length(x$csumlist)) {
     cat("\nSpecific class: ",names(x$csumlist[ic]),"\n")
     print(x$csumlist[[ic]],digits=x$digits)
     cat("\n")
   }
 
-  if(x$gls) {
-    cat("\nComponents partitioned by DME from residual var/covariance after GLS-b fit:\n\n")
+  if(x$fixedgls) {
+    cat("\nComponents partitioned by DME from residual var/covariance after GLS-fixed-effects fit:\n\n")
     for(ic in 1:length(x$gcsumlist)) {
       cat("\nSpecific class: ",names(x$gcsumlist[ic]),"\n")
       print(x$gcsumlist[[ic]],digits=x$digits)
